# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Meta data of line-bot-sdk."""

from __future__ import unicode_literals

__version__ = '1.12.1'
__author__ = 'LINE Corporation'
__copyright__ = 'Copyright 2016, LINE Corporation'
__license__ = 'Apache 2.0'

__all__ = (
    '__version__'
)
